@@ -1,29 +1,37 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
+# License: GPLv3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
+
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-__license__   = 'GPL v3'
-__copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
-__docformat__ = 'restructuredtext en'
-
-import sys, os, textwrap
-from threading import Thread
+import os
+import sys
+import textwrap
 from functools import partial
-from polyglot.builtins import map, unicode_type, range
+from threading import Thread
 
-from PyQt5.Qt import (QPushButton, QFrame, QMenu, QInputDialog, QCheckBox,
-    QDialog, QVBoxLayout, QDialogButtonBox, QSize, QStackedWidget, QWidget,
-    QLabel, Qt, pyqtSignal, QIcon, QTreeWidget, QGridLayout, QTreeWidgetItem,
-    QToolButton, QItemSelectionModel, QCursor, QKeySequence, QSizePolicy)
+from PyQt5.Qt import (
+    QCheckBox, QCursor, QDialog, QDialogButtonBox, QFrame, QGridLayout, QIcon,
+    QInputDialog, QItemSelectionModel, QKeySequence, QLabel, QMenu, QPushButton,
+    QSize, QSizePolicy, QStackedWidget, Qt, QToolButton, QTreeWidget,
+    QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
+)
 
-from calibre.ebooks.oeb.polish.container import get_container, AZW3Container
+from calibre.constants import TOC_DIALOG_APP_UID, islinux, iswindows
+from calibre.ebooks.oeb.polish.container import AZW3Container, get_container
 from calibre.ebooks.oeb.polish.toc import (
-    get_toc, add_id, TOC, commit_toc, from_xpaths, from_links, from_files)
-from calibre.gui2 import Application, error_dialog, gprefs, info_dialog, question_dialog
+    TOC, add_id, commit_toc, from_files, from_links, from_xpaths, get_toc
+)
+from calibre.gui2 import (
+    Application, error_dialog, gprefs, info_dialog, question_dialog, set_app_uid
+)
+from calibre.gui2.convert.xpath_wizard import XPathEdit
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.toc.location import ItemEdit
-from calibre.gui2.convert.xpath_wizard import XPathEdit
+from calibre.ptempfile import reset_base_dir
+from calibre.utils.lock import ExclusiveFile
 from calibre.utils.logging import GUILog
+from polyglot.builtins import map, range, unicode_type
 
 ICON_SIZE = 24
 
@@ -80,7 +88,7 @@ class XPathDialog(QDialog):  # {{{
             if name:
                 saved = self.prefs.get('xpath_toc_settings', {})
                 # in JSON all keys have to be strings
-                saved[name] = {str(i):x for i, x in enumerate(xpaths)}
+                saved[name] = {unicode_type(i):x for i, x in enumerate(xpaths)}
                 self.prefs.set('xpath_toc_settings', saved)
                 self.setup_load_button()
 
@@ -103,7 +111,7 @@ class XPathDialog(QDialog):  # {{{
     def load_settings(self, name):
         saved = self.prefs.get('xpath_toc_settings', {}).get(name, {})
         for i, w in enumerate(self.widgets):
-            txt = saved.get(str(i), '')
+            txt = saved.get(unicode_type(i), '')
             w.edit.setText(txt)
 
     def check(self):
@@ -1117,10 +1125,30 @@ class TOCEditor(QDialog):  # {{{
 # }}}
 
 
+def main(path=None, title=None):
+    # Ensure we can continue to function if GUI is closed
+    os.environ.pop('CALIBRE_WORKER_TEMP_DIR', None)
+    reset_base_dir()
+    if iswindows:
+        # Ensure that all instances are grouped together in the task bar. This
+        # prevents them from being grouped with viewer/editor process when
+        # launched from within calibre, as both use calibre-parallel.exe
+        set_app_uid(TOC_DIALOG_APP_UID)
+
+    with ExclusiveFile(path + '.lock') as wf:
+        override = 'calibre-gui' if islinux else None
+        app = Application([], override_program_name=override)
+        d = TOCEditor(path, title=title)
+        d.start()
+        ret = 1
+        if d.exec_() == d.Accepted:
+            ret = 0
+        wf.write('{}'.format(ret).encode('ascii'))
+    del d
+    del app
+    raise SystemExit(ret)
+
+
 if __name__ == '__main__':
-    app = Application([], force_calibre_style=True)
-    app
-    d = TOCEditor(sys.argv[-1])
-    d.start()
-    d.exec_()
-    del d  # Needed to prevent sigsegv in exit cleanup
+    main(path=sys.argv[-1], title='test')
+    os.remove(sys.argv[-1] + '.lock')
